/* $Id: driver_cf.c,v 1.3 2003/05/02 12:35:10 dijkstra Exp $ */

/*
 * Copyright (c) 2003 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <errno.h>

#include "lcd.h"
#include "error.h"
#include "driver_cf.h"

__BEGIN_DECLS
void setup_heartbeat();
void setup_special_char();
void setup_specialchar(int ch, struct cf_char *);
void setup_vertical_bar();
__END_DECLS

char *driver_name = "CrystalFontz";
int   driver_capabilities = 0;

void 
driver_init(char *serial, int capabilities) 
{
    struct termios tp;

    debug("initialize %s driver @ %s", driver_name, serial);

    driver_capabilities = LCD_CONTRAST | LCD_BACKLIGHT;

    if ((capabilities & LCD_HBAR) || (capabilities & LCD_HEARTBEAT)) {
	fatal("%s: cannot support horizontal bars and heartbeats",
	      driver_name);
    }

    if ((lcd_fd = open(serial, O_RDWR)) == -1) {
	fatal("%s: cannot open serial '%s': %.200s", 
	      driver_name, strerror(errno));
    }

    if (tcgetattr(lcd_fd, &tp) == -1) {
	fatal("%s: cannot get terminal attributes: %.200s",
	      driver_name, strerror(errno));
    }

    if (cfsetospeed(&tp, B9600) == -1) {
	fatal("%s: cannot set output speed: %.200s",
	      driver_name, strerror(errno));
    }
    
    if (cfsetispeed(&tp, B0) == -1) {
	fatal("%s: cannot set input speed: %.200s",
	      driver_name, strerror(errno));
    }

    cfmakeraw(&tp);

    if (tcsetattr(lcd_fd, TCSANOW, &tp) == -1) {
	fatal("%s: cannot not set serial line %s to raw: %.200s",
	      driver_name, serial, strerror(errno));
    }
  
    sleep(1);
    lcd_reboot();

    if (capabilities & LCD_VBAR)
	setup_vertical_bar();

}

void 
driver_reset()
{
    driver_set_contrast(CF_DEFAULT_CONTRAST);
    driver_set_backlight(CF_DEFAULT_BACKLIGHT);

    lcd_printfraw("%c%c%c%c%c%c",
		  CFCMD_HIDE_DISPLAY,
		  CFCMD_HIDE_CURSOR,
		  CFCMD_FORM_FEED,
		  CFCMD_SCROLL_OFF,
		  CFCMD_WRAP_OFF,
		  CFCMD_RESTORE_DISPLAY);
}

void
driver_flush() 
{
    int i;

    for (i=0; i<LCD_Y; i++) {
	lcd_printfraw("%c%c%c%." LCD_XSTR "s",
		      CFCMD_SET_CURSOR_POSITION, 
		      0, i,
		      lcd_framebuffer + (i * LCD_X));
    }
}

void
driver_put_block(int x, int y) 
{
    lcd_set(x, y, CF_CHAR_BLOCK);
}

void 
driver_put_vchar(int x, int y, int percentage)
{
    int unit;
    int i;

    if (percentage > 100) {
	percentage = 100;
    }
    
    if (percentage < 0) {
	percentage = 0;
    }

    if (percentage == 100) {
	driver_put_block(x, y);
	return;
    }

    if (percentage == 0) {
	lcd_set(x, y, 32);
	return;
    }

    unit = 1000/8;

    percentage += 5;
    percentage *= 10;

    i = percentage / unit;

    if (i == 0) {
	lcd_set(x, y, CF_CHAR_SPACE);
	return;
    }

    if (i == 8) {
	driver_put_block(x, y);
	return;
    }

    lcd_set(x, y, CF_CHAR_SPECIAL + i);
}

void
driver_set_contrast(int contrast)
{
    if (contrast > 100) {
	contrast = 100;
    }
    if (contrast < 0) {
	contrast = 0;
    }

    lcd_printfraw("%c%c",
		  CFCMD_CONTRAST_CONTROL,
		  contrast);
}

void
driver_set_backlight(int backlight)
{
    if (backlight > 100) {
	backlight = 100;
    }
    if (backlight < 0) {
	backlight = 0;
    }

    lcd_printfraw("%c%c",
		  CFCMD_BACKLIGHT_CONTROL,
		  backlight);
}

void
setup_specialchar(int ch, struct cf_char *l)
{
    int ch_data[8];
    int i, bit, mask;
    
    if ((ch < 0) || (ch > 7)) {
	fatal("%s: cannot set specialchar %d",
	      driver_name, ch);
    }

    for (i=0; i<8; i++) {
	ch_data[i] = 0;
	mask = 0xF;

	for (bit=0; bit<6; bit++) {
	    ch_data[i] += ((l->line[i] & mask)?(1<<bit):0);
	    mask <<= 4;
	}
    }

    lcd_printfraw("%c%c%c%c%c%c%c%c%c%c",
		  CFCMD_SET_CUSTOM_CHARACTER_BITMAP,
		  ch,
		  ch_data[0], ch_data[1], ch_data[2], ch_data[3],
		  ch_data[4], ch_data[5], ch_data[6], ch_data[7]);
}

void 
setup_vertical_bar() 
{
    int i;
    struct cf_char vbar[] = {
	{{
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	}},
	{{	    
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}},
	{{
	    0x000000,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	    0x111111,
	}}
    };

    for (i=0; i< (sizeof(vbar) / sizeof(struct cf_char)); i++) {
	setup_specialchar(i, &vbar[i]);
    }

    driver_capabilities &= !LCD_HBAR;
    driver_capabilities |= LCD_VBAR;
}
