/* $Id: st_if.c,v 1.2 2003/05/02 12:35:10 dijkstra Exp $ */

/*
 * Copyright (c) 2003 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <string.h>

#include "error.h"
#include "lcd.h"
#include "st_if.h"
#include "stream.h"
#include "xmalloc.h"

void 
if_init(struct stream *st)
{
    struct ps_if ps_if_data[LCD_X];
    
    if (st->data != NULL) {
	warning("%s:%d:st->data != NULL", __FILE__, __LINE__);
	xfree(st->data);
    }

    st->data = xmalloc(sizeof(ps_if_data));
    bzero(st->data, sizeof(ps_if_data));
}

void
if_read(struct stream *st, char *s, size_t maxlen)
{
    struct ps_if  if_data;
    struct ps_if  *last, *curr;
    int data_index, i;
    long unsigned int timestamp, dt, dbytes_in, dbytes_out;

    timestamp = 0;
    i = (st->index + 1) % LCD_X;
    
    /* jump over if:arg: */
    data_index = get_nextarg_index(s, maxlen, 2);

    if (1 != sscanf(&s[data_index], "%lu:", &timestamp))
	return;

    /* jump over timestamp : packets in: packets out */
    data_index += get_nextarg_index(&s[data_index], maxlen - data_index, 3);

    if (2 != sscanf(&s[data_index], "%lu:%lu:", 
		    &if_data.bytes_in, &if_data.bytes_out))
	return;

    /* all data was read succesfully - commit */
    curr = &((struct ps_if *)st->data)[i];
    last = &((struct ps_if *)st->data)[st->index];

    curr->bytes_in   = if_data.bytes_in;
    curr->bytes_out  = if_data.bytes_out;
    curr->timestamp  = timestamp;
    dt = timestamp - last->timestamp;

    /* can only compute difference after 2nd read */
    if (st->lastupdate != 0) {
	dbytes_in        = if_data.bytes_in - last->bytes_in;
	dbytes_out       = if_data.bytes_out - last->bytes_out; 

	/* handle overflows */
	if (last->bytes_in > if_data.bytes_in)
	    dbytes_in += 4294967296;
	if (last->bytes_out > if_data.bytes_out)
	    dbytes_out += 4294967296;
	
	curr->bps_in     = (dbytes_in << 3) / dt;
	curr->bps_out    = (dbytes_out << 3) / dt;

    } 

    st->index = i;
    st->lastupdate = timestamp;
}
/* determine maxima over last <positions> */
void
if_get_max(struct stream *st, double *max_in, double *max_out, int positions)
{
    int c, index;
    struct ps_if *curr;

    *max_in = 0.0;
    *max_out = 0.0;

    if (positions > LCD_X)
	fatal("Requested more positions precision than stored");

    if (positions <= 0) 
	positions = LCD_X;

    for (c = 0; c < positions; c++) {
	index = (st->index - c ) % LCD_X;
	curr = &((struct ps_if *)st->data)[index];
	
	if (curr->bps_in > *max_in)
	    *max_in     = curr->bps_in;

	if (curr->bps_out > *max_out)
	    *max_out     = curr->bps_out;
    }
}
/* Mode IFMODE_INOUT
 *
 * +0123456789012345+
 * 0ifn: 123Ki 123Ko0
 * 1-=-=-=-i-=-=-=-o1
 * +0123456789012345+
 */
void
if_show(struct stream *st, int mode)
{
    double max_din, max_dout, max_dinout, percent;
    struct ps_if *curr;
    int i, ri, bar;

    max_din = 0.0;
    max_dout = 0.0;

    lcd_clear();

    switch (mode) {
    case IFMODE_IN:
	if_get_max(st, &max_din, &max_dout, LCD_X);

	percent = (max_din / 100);
	for (i = 0; i < LCD_X; i++) {
	    ri = (st->index + i + 1) % LCD_X;
	    curr = &((struct ps_if *)st->data)[ri];
	    lcd_vbar(i, 1, 1, (curr->bps_in / percent));
	}
	break;

    case IFMODE_OUT:
	if_get_max(st, &max_din, &max_dout, LCD_X);

	percent = (max_dout / 100);
	for (i = 0; i < LCD_X; i++) {
	    ri = (st->index + i + 1) % LCD_X;
	    curr = &((struct ps_if *)st->data)[ri];
	    lcd_vbar(i, 1, 1, (curr->bps_out / percent));
	}
	break;

    case IFMODE_INOUT:
	if_get_max(st, &max_din, &max_dout, LCD_X/2);
	max_dinout = (max_dout > max_din) ? max_dout : max_din;    
	percent = (max_dinout / 100);

	for (i = 0; i < LCD_X/2 - 1; i++) {
	    ri = (st->index - i) % LCD_X;
	    curr = &((struct ps_if *)st->data)[ri];

	    bar = (int)(curr->bps_in / percent);
	    lcd_vbar((LCD_X/2) - 2 - i, 1, 1, bar);

	    bar = (int)(curr->bps_out / percent);
	    lcd_vbar( LCD_X    - 2 - i, 1, 1, bar);
	}
	
	lcd_printf((LCD_X/2) - 1, 1, "%c", 'i');
	lcd_printf( LCD_X    - 1, 1, "%c", 'o');
	break;

    default:
	fatal("illegal interface display mode selected");
    }

    curr = &((struct ps_if *)st->data)[st->index];
    
    lcd_printf(0, 0, "%4s", st->sargs);
    lcd_printunit(5, 0, 3, 0, curr->bps_in);
    lcd_printf(9, 0, "%c", 'i');
    lcd_printunit(11, 0, 3, 0, curr->bps_out);
    lcd_printf(15, 0, "%c", 'o');
}
